/**
   \file    vector.hh
   \brief   numerical vector class
   \author  Clemens Pechstein
   \date    November 2009
*/
#ifndef __VECTOR_H
#define __VECTOR_H

#include <cmath>
#include <iostream>
#include <cassert>
#include <valarray>

using namespace std;


class Vector
{
private:
  valarray<double> vec_;
  
public:
  Vector (int size=0)
    : vec_(size)
  { }
  
  // assignment
  Vector& operator= (const Vector& v)
  {
    if (vec_.size () != v.vec_.size ())
      {
	vec_.resize (v.vec_.size ());
      }
    vec_ = v.vec_;
    return *this;
  }
  
  // set all elements to value
  Vector& operator= (double value)
  {
    vec_ = value;
    return *this;
  }
  
  // get size
  int size () const
  {
    return (int)vec_.size ();
  }
  
  // resize and initialize with 0
  void resize (int newsize)
  {
    vec_.resize ((size_t)newsize);
  }
  
  // element access
  double operator[] (int i) const { assert (0<=i && i<size()); return vec_[i]; }
  double& operator[] (int i) { assert (0<=i && i<size()); return vec_[i]; }

  void setToZero ();
  
  // print
  void print (ostream& os) const;
  
  // add vector to this one
  Vector& operator+= (const Vector& v);
  
  // sub vector from this one
  Vector& operator-= (const Vector& v);
  
  // multiply by factor
  Vector& operator*= (double factor);

  inline
  Vector operator+ (const Vector& v) const
  {
    Vector res(*this);
    res += v;
    return res;
  }

  inline
  Vector operator- (const Vector& v) const
  {
    Vector res(*this);
    res -= v;
    return res;
  }
  
  inline
  Vector operator* (double factor) const
  {
    Vector res(*this);
    res *= factor;
    return res;
  }

  // Euclidean norm
  double l2norm () const;
  
}; // class Vector


inline
ostream& operator<< (ostream& os, const Vector& v)
{
  v.print (os);
  return os;
}


// Euclidean inner product
double inner_product (const Vector& v, const Vector& w);


inline
Vector operator* (double factor, const Vector& v)
{
  Vector res(v);
  res *= factor;
  return res;
}

#endif // __VECTOR_H

